@@ -447,6 +447,7 @@ def multiple_gapfill(reactions, positive, negative, min_growth_conditions, close
 
     log_and_message("Gap filling from SUBSYSTEMS", stderr=verbose)
     subsystem_reactions = PyFBA.gapfill.suggest_reactions_from_subsystems(modeldata.reactions, reactions,
+                                                                          organism_type=genome_type,
                                                                           threshold=0.5, verbose=verbose)
     added_reactions.append(("subsystems", subsystem_reactions))
     reactions.update(subsystem_reactions)
@@ -500,9 +501,15 @@ def gapfill_multiple_media():
     parser.add_argument('-v', '--verbose', help='verbose output', action='store_true')
     args = parser.parse_args(sys.argv[2:])
 
+    if not os.path.exists(args.reactions):
+        sys.stderr.write(f"FATAL: {args.reactions} does not exist. Please check your files\n")
+        sys.exit(1)
+
     if args.type not in orgtypes:
         sys.exit("Sorry, {} is not a valid organism type".format(args.t))
 
+    log_and_message(f"Running PyFBA with the parameters: {sys.argv}\n", quiet=True)
+
     # read the enzyme data
     # compounds, reactions, enzymes = PyFBA.parse.model_seed.compounds_reactions_enzymes(args.t)
     global modeldata
