@@ -1,5 +1,7 @@
 import sys
 
+import PyFBA.metabolism
+
 
 class Reaction:
     """
@@ -29,14 +31,17 @@ class Reaction:
     =        Bidirectional
     ======   ===========================
 
-    :ivar name: The name of the reaction
+    :ivar rctn_id: The reaction ID
+    :ivar readable_name: The name of the reaction
     :ivar description: A description of the reaction
     :ivar equation: The reaction equation
     :ivar direction: The direction of the reaction (<, =, >, or ?)
-    :ivar left_compounds: A set of compounds on the left side of the reaction
-    :ivar left_abundance: A dict of the compounds on the left and their abundance
-    :ivar right_compounds: The set of compounds on the right side of the equation
-    :ivar right_abundance: A dict of the compounds on the right and their abundance
+    :ivar gfdirection: The possible gapfilled direction
+    :ivar ntdirection: The non-template direction (before correcting for templates)
+    :ivar left_compounds: A set of CompoundWithLocations on the left side of the reaction
+    :ivar left_abundance: A dict of the CompoundWithLocations on the left and their abundance
+    :ivar right_compounds: The set of CompoundWithLocations on the right side of the equation
+    :ivar right_abundance: A dict of the CompoundWithLocations on the right and their abundance
     :ivar lower_bound: The lower bound for the reaction
     :ivar upper_bound: The upper bound for the reaction
     :ivar pLR: The probability the reaction proceeds left to right
@@ -57,29 +62,34 @@ class Reaction:
 
     """
 
-    def __init__(self, name):
+    def __init__(self, rctn_id, readable_name=None, description=None, equation=None, direction=None):
         """
-        Instantiate the reaction
+        Instantiate a reaction
 
-        :param name: The name of the reaction
-        :type name: str
-        :return:
-        :rtype:
+        :param rctn_id: the reaction id
+        :param readable_name: a human readable name. This was refactored from name to make it more unique
+        :param description: a description of the reaction
+        :param equation: the equation for the reaction
+        :param direction: the direction of the reaction
         """
-
-        self.name = name
-        self.description = None
-        self.equation = None
-        self.direction = None
-        self.left_compounds = set()
+        self.id = rctn_id
+        self.model_seed_id = rctn_id
+        self.readable_name = readable_name
+        self.description = description
+        self.equation = equation
+        self.direction = direction
+        self.gfdirection = direction # the gap filling direction
+        self.ntdirection = direction # the non-template driven direction
+        self.left_compounds = set()  # type: set[PyFBA.metabolism.CompoundWithLocation]
         self.left_abundance = {}
-        self.right_compounds = set()
+        self.right_compounds = set()  # type: set[PyFBA.metabolism.CompoundWithLocation]
         self.right_abundance = {}
         self.lower_bound = None
         self.upper_bound = None
         self.pLR = 0
         self.pRL = 0
         self.enzymes = set()
+        self.ec_numbers = []
         self.pegs = set()
         self.deltaG_error = 0
         self.deltaG = 0
@@ -99,7 +109,6 @@ class Reaction:
         right products, but not necessarily the same names or reactions.
         Note that we don't care whether the left and right (the 
         directionality) is the same in our two comparisons
-
         :param other: The other reaction
         :type other: Reaction
         :return: Boolean
@@ -107,17 +116,18 @@ class Reaction:
         """
 
         if isinstance(other, Reaction):
-            return ((self.left_compounds, self.right_compounds) ==
+            return (self.id == other.id or
+                    (self.left_compounds, self.right_compounds) ==
                     (other.left_compounds, other.right_compounds) or
                     (self.left_compounds, self.right_compounds) ==
-                    (other.right_compounds, other.left_compounds))
+                    (other.right_compounds, other.left_compounds)
+                    )
         else:
-            return NotImplemented
+            raise NotImplementedError(f"Comparing Reaction with {type(other)} is not implemented")
 
     def __cmp__(self, other):
         """
         Compare whether two things are the same
-
         :param other: The other reaction
         :type other: Reaction
         :return: an integer, zero if they are the same
@@ -130,42 +140,94 @@ class Reaction:
             else:
                 return 1
         else:
-            return NotImplemented
-
+            raise NotImplementedError(f"Comparing Reaction with {type(other)} is not implemented")
 
     def __ne__(self, other):
         """
         Are these not equal?
-
         :param other: The other reaction
         :type other: Reaction
         :return: Boolean
         :rtype: bool
         """
 
-        result = self.__eq__(other)
-        if result is NotImplemented:
-            return result
+        try:
+            result = self.__eq__(other)
+        except NotImplementedError:
+            return True
         return not result
 
-
     def __hash__(self):
         """
         The hash function is based on the name of the reaction.
-
         :rtype: int
         """
-        return hash(self.name)
-
+        return hash((self.id, self.readable_name))
 
     def __str__(self):
         """
-        The string version of the reaction. Currently we return self.name as the str
-
+        The string version of the reaction.
         :rtype: str
         """
+        if self.readable_name:
+            return f"{self.id}: {self.readable_name}"
+        else:
+            return f"{self.id}: {self.equation}"
+
+    """
+        Since we have complex data structures, we can't just pickle them and unpickle them with aplomb!
+        In fact, this is affecting deep/shallow copy, and we need to ensure that we use copy.deepcopy() 
+        at all times, otherwise the data structures are not copied correctly.
+        
+        These two methods correctly allow us to pickle the data structures. Note that we have
+        CompoundWithLocation objects, and we need both the object and its abundance to correctly create the pickle.
+    """
+
+    def __getstate__(self):
+        """
+        The state that the object is saved or copied as. We override the left/right compounds and abundances
+        with simple arrays of data. This is lossy - we are losing the connections between compounds and data
+        and we probably need to reconstruct that after pickling/unpickling the reactions.
+        :return:
+        """
+        state = self.__dict__.copy()
+        state['left_compounds'] = []
+        state['right_compounds'] = []
+        state['left_abundance'] = {}
+        state['right_abundance'] = {}
+        for l in self.left_compounds:
+            state['left_compounds'].append([l.id, l.name, l.location])
+            state['left_abundance'][f"{l.id} :: {l.name} :: {l.location}"] = self.left_abundance[l]
+        for r in self.right_compounds:
+            state['right_compounds'].append([r.id, r.name, r.location])
+            state['right_abundance'][f"{r.id} :: {r.name} :: {r.location}"] = self.right_abundance[r]
+        return state
+
+
+    def __setstate__(self, state):
+        """
+        Create a new reaction from a saved state. This is from __getstate__ eg. when pickled.
+        :param state: the state that was saved.
+        :return:
+        """
+        left = set()
+        right = set()
+        left_abundance = {}
+        right_abundance = {}
+        for l in state['left_compounds']:
+            c = PyFBA.metabolism.CompoundWithLocation(id=l[0], name=l[1], location=l[2])
+            left.add(c)
+            left_abundance[c] = state['left_abundance'][f"{l[0]} :: {l[1]} :: {l[2]}"]
+        state['left_compounds'] = left
+        state['left_abundance'] = left_abundance
+        for r in state['right_compounds']:
+            c = PyFBA.metabolism.CompoundWithLocation(id=r[0], name=r[1], location=r[2])
+            right.add(c)
+            right_abundance[c] = state['right_abundance'][f"{r[0]} :: {r[1]} :: {r[2]}"]
+        state['right_compounds'] = right
+        state['right_abundance'] = right_abundance
+        self.__dict__.update(state)
 
-        return self.name
 
 
     def set_direction(self, direction):
@@ -181,6 +243,8 @@ class Reaction:
         allowable_directions = {'>', '<', '=', None}
         if direction in allowable_directions:
             self.direction = direction
+            if not self.gfdirection:
+                self.gfdirection = direction
         else:
             sys.stderr.write("Direction: " + str(direction) + " is not a permitted direction. Ignored\n")
             self.direction = None
@@ -190,28 +254,33 @@ class Reaction:
     def add_left_compounds(self, cmpds):
         """
         The compounds on the left are a set of compounds that the reaction typically uses as substrates.
-
         :param cmpds: The compounds that should be added
-        :type cmpds: set
+        :type cmpds: set[PyFBA.metabolism.CompoundWithLocation]
         """
 
         if isinstance(cmpds, set):
-                self.left_compounds.update(cmpds)
+            # choose one element. next(iter(cmpds)) does not remove the element
+            if not isinstance(next(iter(cmpds)), PyFBA.metabolism.CompoundWithLocation):
+                raise TypeError(f"Starting with v.2 reactions need PyFBA.metabolism.CompoundWithLocation objects not {type(next(iter(cmpds)))}")
+            self.left_compounds.update(cmpds)
+        elif isinstance(cmpds, PyFBA.metabolism.CompoundWithLocation):
+            # add a single compound
+            self.left_compounds.add(cmpds)
         else:
-            raise TypeError("Compounds must be a set")
+            raise TypeError("Compounds must be a set of CompoundWithLocation")
 
     def set_left_compound_abundance(self, cmpd, abundance):
         """
         Set the abundance of a compound on the left side of the equation.
 
         :param cmpd: The compound to set the abundance for
-        :type cmpd: Compound
+        :type cmpd: PyFBA.metabolism.CompoundWithLocation
         :param abundance: The amount of that abundance
-        :type abundance: float
+        :type abundance: float | int
         """
 
         if cmpd not in self.left_compounds:
-            raise KeyError(cmpd + " is not in left compounds. Please add it before trying to set the abundance")
+            raise KeyError(f"{cmpd} is not in left compounds. Please add it before trying to set the abundance")
         if isinstance(abundance, float):
             self.left_abundance[cmpd] = abundance
         elif isinstance(abundance, int):
@@ -224,7 +293,7 @@ class Reaction:
         Get the abundance of the compound on the left side of the equation.
 
         :param cmpd: The compound to get the abundance of
-        :type cmpd: Compound
+        :type cmpd: PyFBA.metabolism.CompoundWithLocation
         :return: The compounds abundance
         :rtype: float
         """
@@ -232,7 +301,8 @@ class Reaction:
         if cmpd in self.left_abundance:
             return self.left_abundance[cmpd]
         else:
-            raise KeyError("You do not have " + cmpd + " on the left hand side of the equation")
+            raise KeyError(f"In the reaction {self.readable_name} (reaction id: {self.id}), you do not have" +
+                           f" {cmpd} on the left hand side of the equation: {self.equation}")
 
     def number_of_left_compounds(self):
         """
@@ -247,24 +317,30 @@ class Reaction:
         The compounds on the right are a set of compounds that the reaction typically uses as substrates.
 
         :param cmpds: The compounds that should be added
-        :type cmpds: set
+        :type cmpds: set[PyFBA.metabolism.CompoundWithLocation]
         """
         if isinstance(cmpds, set):
+            # choose one element. next(iter(cmpds)) does not remove the element
+            if not isinstance(next(iter(cmpds)), PyFBA.metabolism.CompoundWithLocation):
+                raise TypeError("Starting with v.2 reactions need PyFBA.metabolism.CompoundWithLocation objects")
             self.right_compounds.update(cmpds)
+        elif isinstance(cmpds, PyFBA.metabolism.CompoundWithLocation):
+            # add a single compound
+            self.right_compounds.add(cmpds)
         else:
-            raise TypeError("Compounds must be a set")
+            raise TypeError("Compounds must be a set of CompoundWithLocation")
 
     def set_right_compound_abundance(self, cmpd, abundance):
         """
         Set the abundance of a compound on the right side of the equation
 
         :param cmpd: The compound to set the abundance for
-        :type cmpd: Compound
+        :type cmpd: PyFBA.metabolism.CompoundWithLocation
         :param abundance: The amount of that abundance
-        :type abundance: float
+        :type abundance: float | int
         """
         if cmpd not in self.right_compounds:
-            raise KeyError(cmpd + " is not in right compounds. " + " Please add it before trying to set the abundance")
+            raise KeyError(f"{cmpd} is not in right compounds. " + " Please add it before trying to set the abundance")
         if isinstance(abundance, float):
             self.right_abundance[cmpd] = abundance
         elif isinstance(abundance, int):
@@ -285,13 +361,12 @@ class Reaction:
         if cmpd in self.right_abundance:
             return self.right_abundance[cmpd]
         else:
-            raise KeyError("You do not have " + str(cmpd) + " on the right hand side of the equation: " +
-                           str(self.equation))
+            raise KeyError(f"In the reaction {self.readable_name} (reaction id: {self.id}), you do not have" +
+                           f" {cmpd} on the right hand side of the equation: {self.equation}")
 
     def number_of_right_compounds(self):
         """
         The number of compounds on the right side of the equation.
-
         :rtype: int
         """
         return len(self.right_compounds)
@@ -299,7 +374,6 @@ class Reaction:
     def all_compounds(self):
         """
         Get all the compounds involved in this reaction.
-
         :return: A set of all the compounds
         :rtype: set
         """
@@ -308,7 +382,6 @@ class Reaction:
     def number_of_compounds(self):
         """
         Get the total number of compounds involved in this reaction.
-
         :rtype: int
         """
         return len(self.all_compounds())
@@ -316,7 +389,6 @@ class Reaction:
     def has(self, cmpd):
         """
         Does this reaction have a compound? Just returns true if the compound is present somewhere in the reaction.
-
         :param cmpd: The compound to test for
         :type cmpd: Compound
         :rtype: bool
@@ -326,7 +398,6 @@ class Reaction:
     def opposite_sides(self, cmpd1, cmpd2):
         """
         Are these two compounds on opposite sides of the reaction?
-
         :param cmpd1: The first compound
         :type cmpd1: Compound
         :param cmpd2: The second compound
@@ -347,7 +418,6 @@ class Reaction:
     def set_probability_left_to_right(self, p):
         """
         Set the probability of the reaction running left to right. Note you can also access this as reaction.pLR
-
         :param p: The probablity
         :type p: float
 
@@ -362,7 +432,6 @@ class Reaction:
     def get_probability_left_to_right(self):
         """
         Get the probability of the reaction running left to right. Note you can also access this as reaction.pLR
-
         :return: The probablity
         :rtype p: float
         """
@@ -371,7 +440,6 @@ class Reaction:
     def set_probability_right_to_left(self, p):
         """
         Set the probability of the reaction running right to left Note you can also access this as reaction.pRL
-
         :param p: The probablity
         :type p: float
         """
@@ -385,13 +453,11 @@ class Reaction:
     def get_probability_right_to_left(self):
         """
         Get the probability of the reaction running right to left. Note you can also access this as reaction.pRL
-
         :return: The probablity
         :rtype p: float
         """
         return self.pRL
 
-
     def add_enzymes(self, enz):
         """
         Add one or more enzymes that completes this reaction.
@@ -407,7 +473,6 @@ class Reaction:
     def has_enzyme(self, enz):
         """
         Check whether an enzyme is involved in this reaction.
-
         :param enz: An Enzyme object
         :type enz: Enzyme
         :return: Whether we have this enzyme
@@ -418,7 +483,6 @@ class Reaction:
     def all_enzymes(self):
         """
         Get all the enzymes involved in this reaction. Returns a set of complex IDs.
-
         :rtype: set
         """
         return self.enzymes
@@ -426,7 +490,6 @@ class Reaction:
     def number_of_enzymes(self):
         """
         Gets the number of enzymes involved in this reaction.
-
         :rtype: int
         """
         return len(self.enzymes)
@@ -434,7 +497,6 @@ class Reaction:
     def add_pegs(self, pegs):
         """
         Add one or more pegs to this reaction. Pegs must be a set.
-
         :param pegs: The pegs to add to the reaction
         :type pegs: set
         """
@@ -446,7 +508,6 @@ class Reaction:
     def has_peg(self, peg):
         """
         Check whether a peg is involved in this reaction.
-
         :param peg: The peg to check for
         :type peg: str
         :rtype: bool
@@ -457,7 +518,6 @@ class Reaction:
         """
         Set the value for delta G (Gibbs free energy) for this reaction. Recall -ve deltaG means the reaction is
         favorable.
-
         :param dg: The delta G of the reaction
         :type dg: float
         """
@@ -471,7 +531,6 @@ class Reaction:
     def get_deltaG(self):
         """
         Get the value for delta G (Gibbs free energy) for this reaction.
-
         :rtype: float
         """
         return self.deltaG
@@ -479,11 +538,11 @@ class Reaction:
     def check_input_output(self):
         """
         Check whether this reaction is an input or output reaction.
-
         This is called when we ask is_input_reaction / is_output_reaction and both inp and outp are False
         """
 
         # do we have external compounds on the left ... then it is an input reaction
+
         for c in self.left_compounds:
             if c.location == 'e':
                 self.inp = True
@@ -527,7 +586,6 @@ class Reaction:
     def is_output_reaction(self):
         """
         Is this an output reaction?
-
         :rtype: bool
         """
         if self.inp is False and self.outp is False:
@@ -556,8 +614,37 @@ class Reaction:
         elif self.direction == '<':
             self.direction = '>'
 
+        # we only need to reverse two gfdirections
+        if self.gfdirection == '>':
+            self.gfdirection = '<'
+        elif self.gfdirection == '<':
+            self.gfdirection = '>'
+
+        if self.lower_bound != None and self.upper_bound != None:
+            lbtemp = 0 - self.lower_bound
+            self.lower_bound = 0 - self.upper_bound
+            self.upper_bound = lbtemp
+
         (self.pLR, self.pRL) = (self.pRL, self.pLR)
         self.deltaG = -self.deltaG
 
+    def add_attribute(self, key, value):
+        """
+        Add an attribute to this class
+        """
+        setattr(self, key, value)
 
+    def get_attribute(self, key):
+        """
+        Retrieve an attribute
+        """
+        return getattr(self, key)
 
+    def reset_bounds(self):
+        """
+        reset the bounds of this reaction. If we are using this in gapfilling, we need to reset the bounds
+        so we can calculate appropriately.
+        :return: None
+        """
+        self.lower_bound = None
+        self.upper_bound = None
\ No newline at end of file
