@@ -1,5 +1,6 @@
 from .read_media import read_media_file, pyfba_media, media_files, correct_media_names, raw_media
-from .rast import read_assigned_functions, roles_of_function, roles_to_subsystem, read_functional_roles, read_features_file
+from .rast import read_assigned_functions, roles_of_function, roles_to_subsystem
+from .rast import read_functional_roles, read_features_file, assigned_functions_set
 from .model_seed import compounds_reactions_enzymes, parse_model_seed_data
 from .SBML import parse_sbml_file
 
